
/******************************************************************************
* MODULE     : striker.gen.cc
* DESCRIPTION: Striker
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rectangles.hh"
#include "Widget/basic_widget.hh"
#include "window.hh"
#include "dir.hh"

#include <stdlib.h>
#include <math.h>

typedef long long int llint;
typedef rectangle  line;
typedef rectangles lines;

#define GAME_WIDTH  600*PIXEL
#define GAME_HEIGHT 500*PIXEL

/******************************************************************************
* Game widget
******************************************************************************/

class game_widget_rep: public basic_widget_rep {
  widget footer;
  int    mode;
  int    who;
  lines  player[2];
  int    nr_done;
  SI     mx1, my1;
  SI     mx2, my2;
  bool   game_over;

public:
  game_widget_rep (widget footer);
  operator tree ();

  void new_game (int type);
  void set_mode (int mode);
  void show_statistics ();

  void handle_get_size (get_size_event ev);
  void handle_repaint (repaint_event ev);
  void handle_keypress (keypress_event ev);
  void handle_mouse (mouse_event ev);
};

/******************************************************************************
* Setup game
******************************************************************************/

game_widget_rep::game_widget_rep (widget footer2):
  basic_widget_rep (footer2->dis, south_west), footer (footer2)
{
  mode     = 2;
  game_over= true;
}

game_widget_rep::operator tree () {
  return "striker";
}

void
game_widget_rep::handle_get_size (get_size_event ev) {
  ev->w= GAME_WIDTH; ev->h= GAME_HEIGHT;
}

void
game_widget_rep::set_mode (int mode2) {
  mode=mode2;
  switch (mode) {
  case 1:
    footer <<
      set_widget ("left", text_widget ("Game mode: any line"));
    break;
  case 2:
    footer <<
      set_widget ("left", text_widget ("Game mode: majority wins"));
    break;
  case 3:
    footer <<
      set_widget ("left", text_widget ("Game mode: bounded length"));
    break;
  }
  footer << emit_update ();
}

/******************************************************************************
* New game
******************************************************************************/

void
game_widget_rep::show_statistics () {
  int n0= N(player[0]), n1= N(player[1]);
  string s;
  if (n0==0) { s= "red wins"; game_over= true; }
  if (n1==0) { s= "blue wins"; game_over= true; }
  if (n0*n1!=0) s= as_string (n0) * " blue / " * as_string (n1) * " red";
  footer << set_widget ("right", text_widget (s));
  footer << emit_update ();
}

lines
starting_game (int nr, int who) {
  switch (nr) {
  case 1: {
    lines ls;
    int i, j;
    int x= who==0? 25*PIXEL: GAME_WIDTH /2;
    int y= who==0? 25*PIXEL: GAME_HEIGHT/2;
    for (i=0; i<5; i++)
      for (j=0; j<5; j++)
	ls= lines (line (i*50*PIXEL+x, j*50*PIXEL+y,
			 i*50*PIXEL+x+40*PIXEL, j*50*PIXEL+y+30*PIXEL), ls);
    return ls;
  }
  case 2: {
    lines ls;
    int i, j;
    for (i=1; i<9; i++)
      for (j=1; j<9; j++) {
	SI cx = i*(GAME_WIDTH /10);
	SI cy = j*(GAME_HEIGHT/10);
	SI dx1, dy1, dx2, dy2;
	do {
	  dx1= (rand () % (GAME_WIDTH /(50*PIXEL))) *5*PIXEL;
	  dy1= (rand () % (GAME_HEIGHT/(50*PIXEL))) *5*PIXEL;
	  dx2= (rand () % (GAME_WIDTH /(50*PIXEL))) *5*PIXEL;
	  dy2= (rand () % (GAME_HEIGHT/(50*PIXEL))) *5*PIXEL;
	} while ((dx1==dx2) && (dy1==dy2));
	ls= lines (line (cx+dx1, cy+dy1, cx+dx2, cy+dy2), ls);
      }
    return ls;
  }
  case 3: {
    lines ls;
    int i, j;
    for (i=1; i<9; i++)
      for (j=1; j<=i; j++) {
	SI x1 = (GAME_WIDTH/10)+ 20*PIXEL*i;
	SI y1 = j*(GAME_HEIGHT/10);
	SI x2 = (GAME_WIDTH/10)+ 20*PIXEL*i;
	SI y2 = j*(GAME_HEIGHT/10)+ (3*GAME_HEIGHT/40);
	if (who==1) {
	  x1= GAME_WIDTH -x1;
	  y1= GAME_HEIGHT-y1;
	  x2= GAME_WIDTH -x2;
	  y2= GAME_HEIGHT-y2;
	}
	ls= lines (line (x1, y1, x2, y2), ls);
      }
    return ls;
  }
  case 4: {
    SI x1= 100*PIXEL, y1=50*PIXEL, x2=50*PIXEL, y2=100*PIXEL;
    if (who==1) {
      x1= GAME_WIDTH- x1;
      y1= GAME_HEIGHT-y1;
      x2= GAME_WIDTH -x2;
      y2= GAME_HEIGHT-y2;
    }
    return lines (line (x1, y1, x2, y2));
  }
  case 5: {
    lines ls;
    int j;
    for (j=1; j<=9; j++) {
      SI x1 = 20*PIXEL;
      SI y1 = j*(GAME_HEIGHT/10);
      SI x2 = 20*PIXEL;
      SI y2 = j*(GAME_HEIGHT/10)+ (3*GAME_HEIGHT/40);
      if (who==1) {
	x1= GAME_WIDTH -x1;
	y1= GAME_HEIGHT-y1;
	x2= GAME_WIDTH -x2;
	y2= GAME_HEIGHT-y2;
      }
      ls= lines (line (x1, y1, x2, y2), ls);
    }
    return ls;
  }
  default:
    fatal_error ("invalid starting game", "starting_game");
    return lines ();
  }
}

void
game_widget_rep::new_game (int type) {
  game_over = false;
  who       = 0;
  nr_done   = 0;
  player[0] = starting_game (type, 0);
  player[1] = starting_game (type, 1);
  this << emit_invalidate_all ();
}

/******************************************************************************
* Repainting
******************************************************************************/

void
game_widget_rep::handle_repaint (repaint_event ev) {
  win->set_color (dis->black);
  win->fill (ev->x1, ev->y1, ev->x2, ev->y2);
  int i;
  for (i=0; i<2; i++) {
    lines ls= player[i];
    while (!nil (ls)) {
      line l= ls->item;
      win->set_color (i==0?dis->blue:dis->red);
      win->set_line_style (2*PIXEL);
      win->line (l->x1, l->y1, l->x2, l->y2);
      ls= ls->next;
    }
  }
  if (nr_done>0) {
    win->set_color (who==0?dis->blue:dis->red);
    win->set_line_style (2*PIXEL);
    win->line (mx1- 5*PIXEL, my1- 5*PIXEL, mx1+ 5*PIXEL, my1+ 5*PIXEL);
    win->line (mx1+ 5*PIXEL, my1- 5*PIXEL, mx1- 5*PIXEL, my1+ 5*PIXEL);
    if (nr_done>1) {
      win->line (mx2- 5*PIXEL, my2- 5*PIXEL, mx2+ 5*PIXEL, my2+ 5*PIXEL);
      win->line (mx2+ 5*PIXEL, my2- 5*PIXEL, mx2- 5*PIXEL, my2+ 5*PIXEL);
    }
  }
}

/******************************************************************************
* Select new points
******************************************************************************/

void
game_widget_rep::handle_mouse (mouse_event ev) {
  string type= ev->type;
  SI     x= ev->x, y= ev->y;

  if (game_over) return;
  if (type == "press-left") {
    if ((mode==3) && (nr_done>=1)) {
      llint dx= x-mx1;
      llint dy= y-my1;
      llint r = 100*PIXEL;
      if (dx*dx+dy*dy > r*r) {
	llint l = (llint) sqrt ((double) (dx*dx+dy*dy));
	dx= (dx*r)/l;
	dy= (dy*r)/l;
	x = mx1+ dx;
	y = my1+ dy;
      }
    }

    x=(((x/PIXEL)+1)/5)*5*PIXEL;
    y=(((y/PIXEL)+1)/5)*5*PIXEL;

    if (nr_done>=1) {
      mx2    = x;
      my2    = y;
      nr_done= 2;
    }
    if (nr_done==0) {
      mx1    = x;
      my1    = y;
      nr_done= 1;
    }
    this << emit_invalidate_all ();
  }
  if (type == "press-right") {
    if (nr_done>0) {
      nr_done --;
      this << emit_invalidate_all ();
    }
  }
}

/******************************************************************************
* Compute intersections
******************************************************************************/

void
swap (llint& i, llint& j) {
  llint temp=i; i=j; j=temp;
}

bool
intersects (line l1, line l2) {
  llint a1=l1->x1, b1=l1->y1, a2=l1->x2, b2=l1->y2;
  llint c1=l2->x1, d1=l2->y1, c2=l2->x2, d2=l2->y2;
  llint num = (b2-b1)*(c2-c1)-(a2-a1)*(d2-d1);
  llint den1= (d2-b2)*(c1-c2)+(a2-c2)*(d1-d2);
  llint den2= (d2-b2)*(a1-a2)+(a2-c2)*(b1-b2);
  if (num<0) { num=-num; den1=-den1; den2=-den2; }
  if (num==0) {
    if ((b1-b2)*(c1-a2)!=(a1-a2)*(d1-b2)) return false;
    if (a1==a2) { swap (a1, b1); swap (a2, b2); swap (c1, d1); swap (c2, d2); }
    if (a1>a2) swap (a1, a2);
    if (c1>c2) swap (c1, c2);
    return (a1<=c2) && (a2>=c1);
  }
  return (den1>=0) && (den1<=num) && (den2>=0) && (den2<=num);
}

lines
remains (lines ls, line l) {
  if (nil (ls)) return ls;
  if (intersects (ls->item, l)) return remains (ls->next, l);
  return lines (ls->item, remains (ls->next, l));
}

bool
common_edge (line l1, line l2) {
  llint a1=l1->x1, b1=l1->y1, a2=l1->x2, b2=l1->y2;
  llint c1=l2->x1, d1=l2->y1, c2=l2->x2, d2=l2->y2;
  llint num = (b2-b1)*(c2-c1)-(a2-a1)*(d2-d1);
  if (num!=0) return false;
  if ((b1-b2)*(c1-a2)!=(a1-a2)*(d1-b2)) return false;
  if (a1==a2) { swap (a1, b1); swap (a2, b2); swap (c1, d1); swap (c2, d2); }
  if (a1>a2) swap (a1, a2);
  if (c1>c2) swap (c1, c2);
  return (a1<c2) && (a2>c1);
}

bool
common_edge (lines ls, line l) {
  if (nil (ls)) return false;
  return common_edge (ls->item, l) || common_edge (ls->next, l);
}

int
nr_intersections (lines ls, line l) {
  if (nil (ls)) return 0;
  return (intersects (ls->item, l)?1:0) + nr_intersections (ls->next, l);
}

/******************************************************************************
* Keyboard events
******************************************************************************/

void
game_widget_rep::handle_keypress (keypress_event ev) {
  string key= ev->key;
  if (key == "1") { new_game (1); }
  if (key == "2") { new_game (2); }
  if (key == "3") { new_game (3); }
  if (key == "4") { new_game (4); }
  if (key == "5") { new_game (5); }
  if (key == "a") { set_mode (1); }
  if (key == "b") { set_mode (2); }
  if (key == "c") { set_mode (3); }

  if (game_over) return;
  if ((key == "return") && (nr_done==2)) {
    if ((mx2!=mx1) || (my2!=my1)) {
      line l (mx1, my1, mx2, my2);
      if (common_edge (player[who], l)) return;
      if (mode>=2) {
	if (mode==2) {
	  if (nr_intersections (player[who], l) <=
	      nr_intersections (player[1-who], l)) return;
	}
	if (mode==3) {
	  if (intersects (player[1-who]->item, l) &&
	      (nr_intersections (player[1-who], l)==1)) return;
	  if (nr_intersections (player[who], l)==0) return;	  
	  if (nr_intersections (player[who], l)+1 <
	      nr_intersections (player[1-who], l)) return;
	}
      }
      
      player[who]  = lines (l, player[who]);
      player[1-who]= remains (player[1-who], l);
      nr_done      = 0;
      who          = 1-who;
      show_statistics ();
      this << emit_invalidate_all ();
    }
  }
}

/******************************************************************************
* Redirection of keyboard events to the game widget
******************************************************************************/

class redirect_widget_rep: public basic_widget_rep {
  widget_rep* kbd_focus;

public:
  redirect_widget_rep (widget w, widget_rep* kbd_focus);
  operator tree ();
  void handle_keypress (keypress_event ev);
  void handle_keyboard_focus (keyboard_focus_event ev);
};

redirect_widget_rep::redirect_widget_rep (widget w, widget_rep* kf):
  basic_widget_rep (w->dis, 1), kbd_focus (kf) { a[0]= w; }

redirect_widget_rep::operator tree () {
  return tuple ("redirect keyboard", a[0], widget(kbd_focus)); }

void
redirect_widget_rep::handle_keypress (keypress_event ev) {
  kbd_focus << ev; }

void
redirect_widget_rep::handle_keyboard_focus (keyboard_focus_event ev) {
  kbd_focus << ev; }

widget
redirect_widget (widget w, widget_rep* kbd_focus) {
  return new redirect_widget_rep (w, kbd_focus);
}

/******************************************************************************
* Setting up the game window
******************************************************************************/

class game_menu_command_rep: public command_rep {
  game_widget_rep* w;
  bool flag;
  int  data;

public:
  game_menu_command_rep (game_widget_rep* w2, bool flag2, int data2):
    w (w2), flag(flag2), data(data2) {}
  void apply () {
    if (flag) w->set_mode (data);
    else w->new_game (data);
  }
};

command
menu_game (game_widget_rep* w, int data) {
  return new game_menu_command_rep (w, false, data);
}

command
menu_mode (game_widget_rep* w, int data) {
  return new game_menu_command_rep (w, true, data);
}

widget
game_widget (display dis) {
  array<widget> footer (3);
  array<string> footer_name (3);
  footer[0]= text_widget ("Game mode: majority wins");
  footer[1]= glue_widget (true, false);
  footer[2]= text_widget ("Welcome to the striker");
  footer_name[0]= "left";
  footer_name[2]= "right";
  widget ftr= horizontal_array (footer, footer_name, 1);

  game_widget_rep* gw = new game_widget_rep (ftr);

  widget game_menu= vertical_menu (array<widget> ());
  game_menu << emit_insert ("Diagonal",
    command_button (text_widget ("Diagonal"), menu_game (gw, 1)));
  game_menu << emit_insert ("Random",
    command_button (text_widget ("Random"), menu_game (gw, 2)));
  game_menu << emit_insert ("Domino",
    command_button (text_widget ("Domino"), menu_game (gw, 3)));
  game_menu << emit_insert ("Corner",
    command_button (text_widget ("Corner"), menu_game (gw, 4)));
  game_menu << emit_insert ("Sides",
    command_button (text_widget ("Sides"), menu_game (gw, 5)));

  widget mode_menu= vertical_menu (array<widget> ());
  mode_menu << emit_insert ("Any line",
    command_button (text_widget ("Any line"), menu_mode (gw, 1)));
  mode_menu << emit_insert ("Majority wins",
    command_button (text_widget ("Majority wins"), menu_mode (gw, 2)));
  mode_menu << emit_insert ("Bounded length",
    command_button (text_widget ("Bounded length"), menu_mode (gw, 3)));

  widget main_menu= horizontal_array (array<widget> ());
  main_menu << emit_insert ("New",
    pulldown_button (text_widget ("New game"), game_menu));
  main_menu << emit_insert ("Mode",
    pulldown_button (text_widget ("Game mode"), mode_menu));

  array<widget> header (2);
  header[0]= main_menu;
  header[1]= glue_widget (true, false);

  array<widget> game (3);
  game[0]= horizontal_array (header);
  game[1]= gw;
  game[2]= ftr;

  return redirect_widget (vertical_list (game), game[1].rep);
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  install_texmacs ();
  install_tex ();
  display dis= open_display (argc, argv);
  window w1= plain_window (game_widget (dis), "striker");
  w1->map ();
  dis->event_loop ();
  close_display (dis);
  return 0;
}

