
/******************************************************************************
* MODULE     : integer.gen.h
* DESCRIPTION: integers based on gmp
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef INTEGER_H
#define INTEGER_H
#include "tree.hh"

#include <gmp.h>

struct integer_rep: public concrete_struct {
  mpz_t I;
  inline integer_rep () { mpz_init (I); }
  inline ~integer_rep () { mpz_clear (I); }
};

class integer {
  CONCRETE(integer);
public:
  integer ();
  integer (int i);
  integer (string s, int base= 10);
  operator string ();
  operator tree ();
};
CONCRETE_CODE(integer);

ostream& operator << (ostream& out, integer i);
integer operator + (integer i, integer j);
integer operator - (integer i);
integer operator - (integer i, integer j);
integer operator * (integer i, integer j);
integer operator / (integer i, integer j);
integer operator % (integer i, integer j);
integer gcd        (integer i, integer j);

inline void operator += (integer& i, integer j) { i=i+j; }
inline void operator -= (integer& i, integer j) { i=i-j; }
inline void operator *= (integer& i, integer j) { i=i*j; }
inline void operator /= (integer& i, integer j) { i=i/j; }
inline void operator %= (integer& i, integer j) { i=i%j; }

int  sign        (integer i);
bool operator == (integer i, integer j);
bool operator != (integer i, integer j);
bool operator <= (integer i, integer j);
bool operator >= (integer i, integer j);
bool operator <  (integer i, integer j);
bool operator >  (integer i, integer j);


#endif // defined INTEGER_H
