
/******************************************************************************
* MODULE     : word.gen.cc
* DESCRIPTION: abstract power word
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef WORD_CC
#define WORD_CC
#include "Mould/word.hh"
#include "array.hh"

template<class G> int
hash (word<G> w) {
  int i, res= 0;
  for (i=0; i<N(w); i++)
    res= (res<<1) ^ (res>>31) ^ w[i];
  return res;
}

template<class G> word<G>
operator * (word<G> u, word<G> v) {
  int i, n1= N(u), n2= N(v), n= n1+n2;
  word<G> w (n);
  for (i=0; i<n1; i++) w[i]=u[i];
  for (i=0; i<n2; i++) w[i+n1]=v[i];
  return w;
}

template<class G> static array<word<G> >
word<G>::homogeneous (int d) {
  // cout << "Generating " << d << "...\n";
  array<word<G> > a;
  int i, j;
  for (i=1; i<d; i++) {
    word<G> pre (1);
    pre[0]= i;
    array<word<G> > b= word<G>::homogeneous (d-i);
    for (j=0; j<N(b); j++) b[j]= pre * b[j];
    a << b;
  }
  if (d==0) {
    word<G> w;
    a << w;
  }
  else {
    word<G> w (1);
    w[0]= d;
    a << w;
  }
  // cout << "Yields " << a << "\n";
  return a;
}

#endif // defined WORD_CC
