
/******************************************************************************
* MODULE     : relaxed.gen.h
* DESCRIPTION: relaxed power series
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef RELAXED_H
#define RELAXED_H
#include "Univariate/series.hh"

template<class C> class implicit_series;

template<class C> class implicit_series_rep : public series_rep<C> {
public:
  int        r;    // one of the above types
  series<C>  eq;   // the equation

private:
  inline implicit_series_rep (int r2): series_rep<C> (RELAXED), r (r2) {
    n    = r;
    value= zealous<C> (r);
  }
  inline ~implicit_series_rep () {}
  inline C next () { return n<r? value[n]: eq[n]; }
  inline operator tree () { return tree ("implicit"); }

public:
  void set_precision (int terms);
  friend class implicit_series<C>;
};

template<class C> class implicit_series {
  EXTEND_NULL_TEMPLATE(series<C>,implicit_series,C);
public:
  implicit_series (int r);
  C& operator () (int n);
  C  operator [] (int n);
  zealous<C> operator () (int start, int end);
};
EXTEND_NULL_TEMPLATE_CODE(series<C>,implicit_series,class,C);

#include "Univariate/Relaxed/compose.cc"
#include "Univariate/Relaxed/multiply.cc"
#include "Univariate/Relaxed/relaxed.cc"
#endif // defined RELAXED_H
