
/******************************************************************************
* MODULE     : replace.gen.cc
* DESCRIPTION: query replace on several files
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "string.hh"
#include "file.hh"
#include "dir.hh"

string what, by;
bool   diagnosis = false;
bool   kill_flag = false;

string notice1= "* BY         : Joris van der Hoeven
* COPYRIGHT  : This software falls under the GNU general public license;
*              see the file 'LICENSE', which is provided with this package.";

string notice2= "* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.";

string notice3= "* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.";

/******************************************************************************
* Replace
******************************************************************************/

void
replace (string file_name) {
  if (what == "") {
    cerr << "\nreplace] error: no action specified\n";
    return;
  }
  string s;
  if (load_string (file_name, s)) {
    cerr << "\nreplace] error: " << file_name << " did not open\n";
    return;
  }

  int match= 0;
  int i, j;
  string r;
  for (i=0; i<N(s); i++) {
    bool flag= (i<(N(s)-N(what)+1));
    if (flag)
      for (j=0; j<N(what); j++)
	if (s[i+j] != what[j]) {
	  flag= false;
	  break;
	}
    if (flag) {
      match++;
      r << by;
      i += N(what)-1;
      if (kill_flag) while ((i < N(s)) && (s[i] != '\n')) i++;
    }
    else r << s[i];
  }
  if (diagnosis) {
    if (match > 0) {
      cout << "replace] " << file_name;
      if (match==1) cout << ": " << match << " match\n";
      else cout << ": " << match << " matches\n";
    }
  }
  else {
    if (match > 0) {
      cerr << "replace] " << file_name;
      if (match==1) cerr << ": " << match << " match\n";
      else cerr << ": " << match << " matches\n";
      if (save_string (file_name, r))
	cerr << "replace] error: file did not open for write\n";
    }
  }
}

void
replace (tree t, string prefix, tree postfixes, tree exceptions) {
  if (is_atomic (t)) {
    string s = t->label;
    int i, n = N(s);
    bool ok= false;
    for (i=0; i<N(postfixes); i++) {
      string postfix= postfixes[i]->label;
      if ((n>=N(postfix)) && (s(n-N(postfix),n) == postfix))
	ok= true;
    }
    for (i=0; i<N(exceptions); i++) {
      string postfix= exceptions[i]->label;
      if ((n>=N(postfix)) && (s(n-N(postfix),n) == postfix))
	ok= false;
    }
    if (ok) replace (prefix * s);
  }
  else {
    int i;
    for (i=1; i<N(t); i++)
      replace (t[i], prefix * t[0]->label * "/", postfixes, exceptions);
  }
}

void
recursive_replace (string dir, tree postfixes, tree exceptions) {
  bool flag;
  tree t= directory_tree (dir, flag);
  if (flag)
    cerr << "replace] error: directory " << dir << " could not be opened\n";
  else replace (t, "", postfixes, exceptions);
}

void
recursive_replace (string dir, tree postfixes) {
  recursive_replace (dir, postfixes, tree (TUPLE));
}

/******************************************************************************
* Version number
******************************************************************************/

void
upgrade_version () {
  cout << "--------------------------------------------------------------\n";
  cout << "Upgrading links\n";
  cout << "--------------------------------------------------------------\n";
  cout << "rm -f $HOME/texmacs/distr\n";
  cout << "rm -f $HOME/texmacs/src\n";
  cout << "rm -f $HOME/texmacs/web\n";
  cout << "rm -f $HOME/texmacs/misc\n";
  
  cout << "ln -s $HOME/distr/TeXmacs-" << by << "-src/TeXmacs-" << by
       << " $HOME/texmacs/distr\n";
  cout << "ln -s $HOME/distr/TeXmacs-" << by << "-src"
       << " $HOME/texmacs/src\n";
  cout << "ln -s $HOME/distr/TeXmacs-" << by << "-web"
       << " $HOME/texmacs/web\n";
  cout << "ln -s $HOME/distr/TeXmacs-" << by << "-misc"
       << " $HOME/texmacs/misc\n";

  if (!diagnosis) {
    system ("rm -f $HOME/texmacs/distr");
    system ("rm -f $HOME/texmacs/src");
    system ("rm -f $HOME/texmacs/web");
    system ("rm -f $HOME/texmacs/misc");

    system ("ln -s $HOME/distr/TeXmacs-" * by * "-src/TeXmacs-" * by *
	    " $HOME/texmacs/distr");
    system ("ln -s $HOME/distr/TeXmacs-" * by * "-src" *
	    " $HOME/texmacs/src");
    system ("ln -s $HOME/distr/TeXmacs-" * by * "-web" *
	    " $HOME/texmacs/web");
    system ("ln -s $HOME/distr/TeXmacs-" * by * "-misc" *
	    " $HOME/texmacs/misc");
  }
}

void
replace_version () {
  tree project_suffixes (TUPLE);
  tree project_exceptions (TUPLE, "changes.tex");
  tree extra_suffixes (TUPLE);
  tree extra_exceptions (TUPLE, "Changes.html");
  project_suffixes << ".h" << ".c" << ".hh" << ".cc"
		   << ".in" << ".1" << ".spec" << ".tex";
  extra_suffixes << ".h" << ".c" << ".hh" << ".cc"
		 << "Makefile" << ".html";

  cout << "--------------------------------------------------------------\n";
  cout << "Version " << what << " ---> " << by << "\n";
  cout << "--------------------------------------------------------------\n";
  recursive_replace ("/home/vdhoeven/texmacs/src",
		     project_suffixes, project_exceptions);
  recursive_replace ("/home/vdhoeven/texmacs/web",
		     extra_suffixes, extra_exceptions);
  recursive_replace ("/home/vdhoeven/texmacs/misc",
		     extra_suffixes, extra_exceptions);

  cout << "--------------------------------------------------------------\n";
  cout << "Upgrading directories\n";
  cout << "--------------------------------------------------------------\n";
  cout << "mv $HOME/distr/TeXmacs-" << what << "-src/TeXmacs-" << what
       << " $HOME/distr/TeXmacs-" << what << "-src/TeXmacs-" << by << "\n";
  cout << "mv $HOME/distr/TeXmacs-" << what << "-src"
       << " $HOME/distr/TeXmacs-" << by << "-src\n";
  cout << "mv $HOME/distr/TeXmacs-" << what << "-web"
       << " $HOME/distr/TeXmacs-" << by << "-web\n";
  cout << "mv $HOME/distr/TeXmacs-" << what << "-misc"
       << " $HOME/distr/TeXmacs-" << by << "-misc\n";

  if (!diagnosis) {
    system ("mv $HOME/distr/TeXmacs-" * what * "-src/TeXmacs-" * what *
	    " $HOME/distr/TeXmacs-" * what * "-src/TeXmacs-" * by);
    system ("mv $HOME/distr/TeXmacs-" * what * string ("-src") *
	    " $HOME/distr/TeXmacs-" * by * "-src");
    system ("mv $HOME/distr/TeXmacs-" * what * string ("-web") *
	    " $HOME/distr/TeXmacs-" * by * "-web");
    system ("mv $HOME/distr/TeXmacs-" * what * string ("-misc") *
	    " $HOME/distr/TeXmacs-" * by * "-misc");
  }

  upgrade_version ();
}

/******************************************************************************
* Mathemagix version number
******************************************************************************/

void
upgrade_mmx_version () {
  cout << "--------------------------------------------------------------\n";
  cout << "Upgrading links\n";
  cout << "--------------------------------------------------------------\n";
  cout << "rm -f $HOME/mathemagix/src\n";
  cout << "rm -f $HOME/mathemagix/web\n";
  
  cout << "ln -s $HOME/distr/Mathemagix-" << by << "-src"
       << " $HOME/mathemagix/src\n";
  cout << "ln -s $HOME/distr/Mathemagix-" << by << "-web"
       << " $HOME/mathemagix/web\n";

  if (!diagnosis) {
    system ("rm -f $HOME/mathemagix/src");
    system ("rm -f $HOME/mathemagix/web");

    system ("ln -s $HOME/distr/Mathemagix-" * by * "-src" *
	    " $HOME/mathemagix/src");
    system ("ln -s $HOME/distr/Mathemagix-" * by * "-web" *
	    " $HOME/mathemagix/web");
  }
}

void
replace_mmx_version () {
  tree project_suffixes (TUPLE);
  tree project_exceptions (TUPLE, "changes.tex");
  tree extra_suffixes (TUPLE);
  tree extra_exceptions (TUPLE, "Changes.html");
  project_suffixes << ".h" << ".c" << ".hh" << ".cc"
		   << ".in" << ".1" << ".spec" << ".tex";
  extra_suffixes << ".h" << ".c" << ".hh" << ".cc"
		 << "Makefile" << ".html";

  cout << "--------------------------------------------------------------\n";
  cout << "Mathemagix version " << what << " ---> " << by << "\n";
  cout << "--------------------------------------------------------------\n";
  recursive_replace ("/home/vdhoeven/mathemagix/src",
		     project_suffixes, project_exceptions);
  recursive_replace ("/home/vdhoeven/mathemagix/web",
		     extra_suffixes, extra_exceptions);

  cout << "--------------------------------------------------------------\n";
  cout << "Upgrading directories\n";
  cout << "--------------------------------------------------------------\n";
  cout << "mv $HOME/distr/Mathemagix-" << what << "-src"
       << " $HOME/distr/Mathemagix-" << by << "-src\n";
  cout << "mv $HOME/distr/Mathemagix-" << what << "-web"
       << " $HOME/distr/Mathemagix-" << by << "-web\n";

  if (!diagnosis) {
    system ("mv $HOME/distr/Mathemagix-" * what * string ("-src") *
	    " $HOME/distr/Mathemagix-" * by * "-src");
    system ("mv $HOME/distr/Mathemagix-" * what * string ("-web") *
	    " $HOME/distr/Mathemagix-" * by * "-web");
  }

  upgrade_mmx_version ();
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  if (argc<2) {
    cout << "Options for replace :\n";
    cout << "  -action <what> <by>      replace <what> <by>\n";
    cout << "  -copyright               replace old copyright notice\n";
    cout << "  -upgrade <to>            upgrade version\n";
    cout << "  -version <what> <by>     modify version\n";
    cout << "  -mmx-upgrade <to>        upgrade mathemagix version\n";
    cout << "  -mmx-version <what> <by> modify mathemagix version\n";
    cout << "  -in <files>              specify the files\n";
    cout << "  -all                     recursively, in all c and c++ files\n";
    cout << "  -diagnosis               only display a diagnosis\n";
    return 0;
  }

  int i;
  for (i=1; i<argc; i++) {
    string s= argv[i];
    if ((s == "-action") && ((i+2) < argc)) {
      what      = argv[++i];
      by        = argv[++i];
      kill_flag = false;
      continue;
    }
    if (s == "-copyright") {
      what      = notice2;
      by        = notice3;
      kill_flag = false;
      continue;
    }
    if ((s == "-upgrade") && ((i+1) < argc)) {
      by= argv[++i];
      kill_flag= false;
      upgrade_version ();
      continue;
    }
    if ((s == "-version") && ((i+2) < argc)) {
      what= argv[++i];
      by  = argv[++i];
      kill_flag= false;
      replace_version ();
      continue;
    }
    if ((s == "-mmx-upgrade") && ((i+1) < argc)) {
      by= argv[++i];
      kill_flag= false;
      upgrade_mmx_version ();
      continue;
    }
    if ((s == "-mmx-version") && ((i+2) < argc)) {
      what= argv[++i];
      by  = argv[++i];
      kill_flag= false;
      replace_mmx_version ();
      continue;
    }
    if (s == "-in") {
      while (((i+1) < argc) && (argv[i+1][0] != '-'))
	replace (string (argv[++i]));
      continue;
    }
    if (s == "-all") {
      recursive_replace (".", tuple (".h", ".c", ".hh", ".cc"));
      continue;
    }
    if (s == "-diagnosis") {
      diagnosis= true;
      continue;
    }
    cerr << "replace] error: unknown option " << s << "\n";
  }
}

