
/******************************************************************************
* MODULE     : xtexfont.gen.cc
* DESCRIPTION: Display tex fonts on screen
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "dir.hh"
#include "font.hh"
#include "window.hh"
#include "Widget/basic_widget.hh"
#include "font.hh"

/******************************************************************************
* The widget
******************************************************************************/

class font_widget_rep: public basic_widget_rep {
  int  sfactor;
  int  pixel;
  font text_fn;
  font fn;

  SI hleft[17], hright[17];
  SI vbot [17], vtop  [17];
  SI sum_x[18], sum_y [18];

public:
  font_widget_rep (display dis, int sf, font text_fn, font fn);
  operator tree ();
  void handle_get_size (get_size_event ev);
  void handle_repaint (repaint_event ev);
};

/******************************************************************************
* Setup game
******************************************************************************/

font_widget_rep::font_widget_rep (display dis, int sfactor2, font f1, font f2):
  basic_widget_rep (dis), sfactor (sfactor2), pixel (sfactor2*PIXEL),
  text_fn (f1), fn (f2)
{
  int x, y;
  text_extents ex;
  hleft[0]= hright[0]= vbot[0]= vtop[0]= 0;

  for (x=1; x<17; x++) {
    char c= 47+x;
    if (x>=11) c= 54+ x;
    text_fn->get_extents (string (c), ex);
    hleft [x]= ex->x3;
    hright[x]= ex->x4;
    vbot[0]= vbot[0] <? ex->y3;
    vtop[0]= vtop[0] >? ex->y4;
  }

  for (y=1; y<17; y++) {
    char c= 47+y;
    if (y>=11) c= 54+ y;
    text_fn->get_extents (string (c), ex);
    hleft [0]= hleft [0] <? ex->x3;
    hright[0]= hright[0] >? ex->x4;
    vbot[y]  = ex->y3;
    vtop[y]  = ex->y4;
  }

  for (y=1; y<17; y++)
    for (x=1; x<17; x++) {
      char c= (y-1)*16 + (x-1);
      fn->get_extents (string (c), ex);
      /*
      cout << "logical " 
	   << (ex->x1>>8) << ", " << (ex->y1>>8) << " : "
	   << (ex->x2>>8) << ", " << (ex->y2>>8) << "\n";
      cout << "ink "
	   << (ex->x3>>8) << ", " << (ex->y3>>8) << " : "
	   << (ex->x4>>8) << ", " << (ex->y4>>8) << "\n";
	   */
      hleft [x]= hleft [x] <? ex->x3;
      hright[x]= hright[x] >? ex->x4;
      vbot  [y]= vbot[y] <? ex->y3;
      vtop  [y]= vtop[y] >? ex->y4;
    }

  sum_x[0]= 0;
  for (x=0; x<17; x++) {
    hleft [x] -= 2*pixel;
    hright[x] += 2*pixel;
    sum_x[x+1] = sum_x[x]+ hright[x]- hleft[x];
  }

  sum_y[0]= 0;
  for (y=0; y<17; y++) {
    vbot[y]   -= 2*pixel;
    vtop[y]   += 2*pixel;
    sum_y[y+1] = sum_y[y]- (vtop[y]- vbot[y]);
  }
}

font_widget_rep::operator tree () {
  return "xtexfont";
}

void
font_widget_rep::handle_get_size (get_size_event ev) {
  ev->w= sum_x[17]/sfactor;
  ev->h= -sum_y[17]/sfactor;
  abs_round (ev->w, ev->h);
}

/******************************************************************************
* Repainting
******************************************************************************/

void
font_widget_rep::handle_repaint (repaint_event ev) {
  int x, y;
  win->set_background (dis->white);
  win->clear (ev->x1, ev->y1, ev->x2, ev->y2);
  win->set_color (dis->black);
  win->set_shrinking_factor (sfactor);

  for (x=1; x<17; x++) {
    char c= 47+x;
    if (x>=11) c= 54+ x;
    win->line (sum_x[x], sum_y[17], sum_x[x], 0);
    text_fn->draw (win, string (c), sum_x[x]- hleft[x], sum_y[0]- vtop[0]);
  }

  for (y=1; y<17; y++) {
    char c= 47+y;
    if (y>=11) c= 54+ y;
    win->line (0, sum_y[y], sum_x[17], sum_y[y]);
    text_fn->draw (win, string (c), sum_x[0]- hleft[0], sum_y[y]- vtop[y]);
  }

  for (y=1; y<17; y++)
    for (x=1; x<17; x++) {
      char c= (y-1)*16 + (x-1);
      fn->draw (win, string (c), sum_x[x]- hleft[x], sum_y[y]- vtop[y]);
    }

  win->set_shrinking_factor (1);
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  if ((argc<3) || (argc>6)) {
    cout << "Usage: xtexfont <family> <size>\n";
    cout << "       xtexfont <family> <size> <dpi>\n";
    cout << "       xtexfont <family> <size> <dpi> <sfactor>\n";
    cout << "       xtexfont <family> <size> <dpi> <sfactor> <design size>\n";
    return 0;
  }
  string name= argv[1];
  int size   = as_int (string (argv[2]));
  int dpi    = argc>3? as_int (string (argv[3])): 600;
  int sfactor= argc>4? as_int (string (argv[4])): 5;
  int dsize  = argc>5? as_int (string (argv[5])): 10;

  int i;
  string win_name ("xtexfont");
  for (i=1; i<argc; i++)
    win_name= win_name * string (" ") * string (argv[i]);
  char* _win_name= as_charp (win_name);

  install_texmacs ();
  install_tex ();
  debug (0, true);
  debug (2, true);
  display dis= open_display (argc, argv);
  font text_fn= tex_font (dis, "cmr", size, dpi, 10);
  font fn     = tex_font (dis, name, size, dpi, dsize);
  window w1=
    plain_window (new font_widget_rep (dis, sfactor, text_fn, fn), _win_name);
  w1->map ();
  dis->event_loop ();
  close_display (dis);

  delete[] _win_name;
  return 0;
}
