
/******************************************************************************
* MODULE     : webcount.gen.cc
* DESCRIPTION: query webcount on several files
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "string.hh"
#include "array.hh"
#include "file.hh"
#include "dir.hh"
#include "hashmap.hh"
#include "iterator.hh"

#include "merge_sort.hh"

/******************************************************************************
* Webcount
******************************************************************************/

void
webcount (string s) {
  hashmap<string,int> counts (0);
  int i, n= N(s), arg=1;
  string page;
  for (i=0; i<n; i++) {
    int start= i, prev;
    while ((i<n) && ((s[i]!='\n') && (s[i]!=' '))) i++;
    string r= s (start, i);
    if (arg == 7) page= r;
    if ((N(r)>=9) && (r(0,9)=="/ftp_home")) page= r;
    arg++;
    // cout << r << ", " << page << "\n";
    if ((i<n) && (s[i]=='\n')) {
      prev= counts[page];
      counts (page)= prev + 1;
      arg=1;
    }
  }

  array<string> a;
  iterator<string> it= iterate (counts);
  while (it->busy ()) a << it->next ();
  merge_sort (a);

  for (i=0; i<N(a); i++)
    cout << counts (a[i]) << "\t" << a[i] << "\n";
}

void
webcount (tree t, string prefix="") {
  if (N(t) == 0) {
    string s= t->label;
    int    n= N(s);
    if ((n>2) && (s(n-2,n) == ".c")) webcount (prefix * s);
    if ((n>2) && (s(n-2,n) == ".h")) webcount (prefix * s);
    if ((n>3) && (s(n-3,n) == ".cc")) webcount (prefix * s);
  }
  else {
    int i;
    for (i=0; i<N(t); i++) webcount (t[i], prefix * t->label * "/");
  }
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  int i;
  for (i=1; i<argc; i++) {
    string file_name= argv[i], s;
    if (load_string (file_name, s))
      cerr << "webcount: file " << file_name << " not found\n";
    webcount (s);
  }
  return 0;
}

